#!/usr/bin/perl

use strict;
use warnings;

use Test::More 'no_plan';

use ok 'XML::LibXSLT::Easy';
use ok 'XML::LibXSLT::Easy::Batch';

use ok 'XML::LibXSLT::Easy::CLI';
use ok 'XML::LibXSLT::Easy::Batch::CLI';

